#
#  Build GCC
#

DESC="Build GCC for PowerPC"

#  Source and patches
GCCVER=4.1.1
SRC=gcc-$GCCVER.tar.bz2
NEWLIBVER=1.14.0
NEWLIB=newlib-$NEWLIBVER.tar.gz
PATCHES=gcc-patches
TARGET=powerpc-eabi

LANGS="c,c++"
CFLAGS="-O2"
CXXFLAGS="-O2"

. common.sh

BLDDIR=$BLDDIR/bld_gcc
GCC_SRC_DIR=$BLDDIR/gcc
SYSROOT=$RELDIR/$TARGET
BINDIR=$RELDIR/bin
SRCDIR=$SRCDIR/gcc
LIB_SRCDIR=$SRCDIR/libs
LIB_RELDIR=$RELDIR/lib/gcc/powerpc-eabi/$GCCVER
#  Only multilibs which do not support hardware FP
MULTILIBS=". v4fxtuning 440"

echo "Building PowerPC GCC"
echo "Current directory is $CURDIR"
echo "Platform is $PLATFORM"

rm -rf $BLDDIR/gcc
rm -rf $BLDDIR/newlib
rm -rf $BLDDIR/build
rm -rf $BLDDIR/log
mkdir -p $BLDDIR/build
mkdir -p $BLDDIR/log

if [ ! -d $RELDIR ]; then mkdir -p $RELDIR; fi

errors=0

#  Unpack and patch sources
echo "Unpacking gcc sources"
(cd $BLDDIR; tar xfj $SRCDIR/$SRC; mv gcc-$GCCVER gcc)

echo "Unpacking newlib sources"
(cd $BLDDIR; tar xfz $SRCDIR/$NEWLIB; mv newlib-$NEWLIBVER newlib)

# Read and apply patches in proper directory
while read DIR PATCH COMMENT; do
   case $DIR in
      \#*) ;;
      stop)
	  echo "stop encountered -- terminating patches"
	  break
	  ;;
      *)  
          echo -n "Applying patch $PATCH"
          cd $BLDDIR/$DIR
	  if [ -f $SRCDIR/$PATCH ]; then
            patch -F0 -p1 < $SRCDIR/$PATCH > $BLDDIR/log/$PATCH.log 2>&1 
            rc=$?
            echo " -- rc=$rc"
            if [ $rc != 0 ]; then errors=1; fi
	  else
	    echo " -- MISSING"
	    errors=1
	  fi
	  ;;
   esac
done < $SRCDIR/$PATCHES

# Link Newlib into gcc source directory
ln -s $BLDDIR/newlib/newlib $BLDDIR/gcc/newlib
ln -s $BLDDIR/newlib/libgloss $BLDDIR/gcc/libgloss

if [ $errors != 0 ]; then 
   echo ""
   echo "Exiting because of previous errors"
   echo ""
   exit
fi

# Set Xilinx version 
echo $XILINX_VERSION >> $BLDDIR/gcc/gcc/DEV-PHASE

#  Create build directory
cd $BLDDIR/build

PATH=$BINDIR:$PATH

echo -n "Configuring GCC"

../gcc/configure                                \
	--target=powerpc-eabi			\
	--prefix=$RELDIR 			\
	--program-prefix="powerpc-eabi-" 	\
	--with-newlib				\
	--enable-languages=$LANGS		\
	--with-board=xilinx			\
	--enable-multilib			\
	--enable-optimize-memory		\
	--disable-libssp			\
	--disable-newlib-atexit-dynamic-alloc	\
	--with-cpu=405				\
        --with-sysroot=$SYSROOT                 \
		> $BLDDIR/log/gcc-config.log 2>&1 
   echo " -- rc=$?"

if [ $NOBUILD = 0 ]; then
  echo -n "Compiling GCC"
  $MAKE all CFLAGS="$CFLAGS" CXXFLAGS="$CXXFLAGS" 	\
		> $BLDDIR/log/gcc-make.log 2>&1 
  echo -n "   make -- rc=$?"
  $MAKE install CFLAGS="$CFLAGS" CXXFLAGS="$CXXFLAGS"	\
		> $BLDDIR/log/gcc-install.log 2>&1
  echo "   install -- rc=$?"

  #  Copy Xilinx libraries to release lib dirs
  cp $LIB_SRCDIR/* $LIB_RELDIR
  for MLIB in $MULTILIBS 
  do
    cp $LIB_SRCDIR/* $LIB_RELDIR/$MLIB
  done
fi

if [ $NOCLEAN = 0 ]; then
  rm -rf $BLDDIR
fi

echo "PowerPC GCC Build Done"
